const axios = require('axios')
const { isStringFull } = require('../utils/common')
const { API_ENDPOINT } = require('../constant/api')

class HttpService {
  /**
   * Constructor
   *
   * @param {string} [baseURL] the url
   * @param {axios.AxiosRequestConfig} [configs] the request configs
   * @returns {axios.AxiosResponse}
   */
  constructor(baseURL, configs) {
    baseURL = isStringFull(baseURL) ? baseURL : API_ENDPOINT
    this.instance = axios.create({
      baseURL,
      headers: { 'Content-Type': 'application/json' },
      ...(configs || {}),
    })
  }

  /**
   * GET
   *
   * @param {object} params parameters
   * @param {string} params.url the url
   * @param {axios.AxiosRequestConfig} [params.configs] the request configs
   * @returns {Promise<axios.AxiosResponse>}
   */
  async get({ url, configs }) {
    return this.instance.get(url, configs)
  }

  /**
   * POST
   *
   * @param {object} params parameters
   * @param {string} params.url the url
   * @param {any} [params.data] the request data
   * @param {axios.AxiosRequestConfig} [params.configs] the request configs
   * @returns {Promise<axios.AxiosResponse>}
   */
  async post({ url, data, configs }) {
    return this.instance.post(url, data, configs)
  }

  /**
   * PUT
   *
   * @param {object} params parameters
   * @param {string} params.url the url
   * @param {any} [params.data] the request data
   * @param {axios.AxiosRequestConfig} [params.configs] the request configs
   * @returns {Promise<axios.AxiosResponse>}
   */
  async put({ url, data, configs }) {
    return this.instance.put(url, data, configs)
  }

  /**
   * DELETE
   *
   * @param {object} params parameters
   * @param {string} params.url the url
   * @param {axios.AxiosRequestConfig} [params.configs] the request configs
   * @returns {Promise<axios.AxiosResponse>}
   */
  async delete({ url, configs }) {
    return this.instance.delete(url, configs)
  }
}

module.exports = {
  HttpService,
}
