/**
 * @namespace hazama-lib/Bundle
 * @memberof hazama-lib
 * @alias Bundle
 */

/**
 * @namespace hazama-lib/ERC20/ERC721/ERC1155
 * @memberof hazama-lib
 * @alias hazama-lib/ERC20/ERC721/ERC1155
 */

/**
 * @namespace hazama-lib/Others
 * @memberof hazama-lib
 * @alias hazama-lib/Others
 */

/**
 * Options for deploying an Multiwrap contract
 *
 * @typedef {Object} BundleContractDeployMetadata
 * @prop {string} name name of the contract
 * @prop {string} [description] Optional description of the contract
 * @prop {any} [image] Optional image for the contract
 * @prop {string} [external_link] Optional url for the contract
 * @prop {string} [symbol] Symbol for the NFTs
 * @prop {Array.<string>} [trusted_forwarders] Custom gasless trusted forwarder addresses
 * @prop {string} [fee_recipient] The address that will receive the proceeds from secondary sales (royalties)
 * @prop {number} [seller_fee_basis_points] The percentage (in basis points) of royalties for secondary sales
 */

/**
 * Signer to be used to sign messages and transactions
 *
 * @example
 * {
 *   data?: ethers.providers.JsonRpcSigner
 *   error?: Error
 *   isIdle: boolean
 *   isLoading: boolean
 *   isFetching: boolean
 *   isSuccess: boolean
 *   isError: boolean
 *   isFetched: boolean
 *   isRefetching: boolean
 *   status: 'idle' | 'error' | 'loading' | 'success'
 * }
 * @typedef {Object} Signer
 * @prop {any} key:{string} properties of Signer
 */

/**
 * BigNumber
 * @private
 * @typedef {Object} BigNumber
 * @prop {string} _hex
 * @prop {boolean} _isBigNumber
 */

/**
 * Bytes
 * @private
 * @typedef {Object} Bytes
 * @prop {number} length
 * @prop {number} n:{number}
 */

/**
 * BigNumberish
 * @typedef {(BigNumber|Bytes|bigint|string|number)} BigNumberish
 */

/**
 * Pagination Parameters
 * @typedef {Object} QueryAllParams
 * @prop {BigNumberish} [start=0] the index to start from (default: 0)
 * @prop {BigNumberish} [count=100] how many items to return (default: 100)
 */

/**
 * Amount
 * @typedef {(string|number)} Amount
 */

/**
 * Represents a certain amount of ERC20 tokens that can be wrapped
 * @typedef {Object} ERC20Wrappable
 * @prop {string} contractAddress contract address
 * @prop {Amount} quantity quantity
 */

/**
 * Represents a ERC721 NFT that can be wrapped
 * @typedef {Object} ERC721Wrappable
 * @prop {string} contractAddress contract address
 * @prop {BigNumberish} tokenId token id
 */

/**
 * Represents a certain amount of ERC1155 NFTs that can be wrapped
 * @typedef {Object} ERC1155Wrappable
 * @prop {string} contractAddress contract address
 * @prop {Amount} quantity quantity
 * @prop {BigNumberish} tokenId token id
 */

/**
 * Represents a ERC20 token output
 * @typedef {Object} ERC20WrappableOutput
 * @prop {string} contractAddress contract address
 * @prop {Amount} quantity quantity
 * @prop {string} contractName contract name
 * @prop {string} tokenName token name
 */

/**
 * Represents a ERC721 NFT output
 * @typedef {Object} ERC721WrappableOutput
 * @prop {string} contractAddress contract address
 * @prop {BigNumberish} tokenId token id
 * @prop {string} contractName contract name
 * @prop {string} tokenName token name
 */

/**
 * Represents a ERC1155 NFT output
 * @typedef {Object} ERC1155WrappableOutput
 * @prop {string} contractAddress contract address
 * @prop {Amount} quantity quantity
 * @prop {BigNumberish} tokenId token id
 * @prop {string} contractName contract name
 * @prop {string} tokenName token name
 */

/**
 * Input for wrapping any number of tokens
 *
 * @example
 * {
 *   erc20Tokens: [
 *     {
 *       contractAddress: "0x...",
 *       quantity: "0.8",
 *     },
 *   ],
 *   erc721Tokens: [
 *     {
 *       contractAddress: "0x...",
 *       tokenId: "0",
 *     },
 *   ],
 *   erc1155Tokens: [
 *     {
 *       contractAddress: "0x...",
 *       tokenId: "1",
 *       quantity: "2",
 *     },
 *   ],
 * }
 * @typedef {Object} TokensToWrap
 * @prop {ERC20Wrappable[]} [erc20Tokens] list of ERC20 tokens
 * @prop {ERC721Wrappable[]} [erc721Tokens] list of ERC721 tokens
 * @prop {ERC1155Wrappable[]} [erc1155Tokens] list of ERC1155 tokens
 */

/**
 * Output for the contents of a bundle wrapped token
 *
 * @example
 * {
 *   erc20Tokens: [
 *     {
 *       contractAddress: "0x...",
 *       quantity: "0.8",
 *     },
 *   ],
 *   erc721Tokens: [
 *     {
 *       contractAddress: "0x...",
 *       tokenId: "0",
 *     },
 *   ],
 *   erc1155Tokens: [
 *     {
 *       contractAddress: "0x...",
 *       tokenId: "1",
 *       quantity: "2",
 *     },
 *   ],
 * }
 * @typedef {Object} WrappedTokens
 * @prop {ERC20Wrappable[]} erc20Tokens list of ERC20 tokens
 * @prop {ERC721Wrappable[]} erc721Tokens list of ERC721 tokens
 * @prop {ERC1155Wrappable[]} erc1155Tokens list of ERC1155 tokens
 */

/**
 * Output for the contents of a bundle wrapped token
 *
 * @example
 * {
 *   erc20Tokens: [
 *     {
 *       contractAddress: "0x...",
 *       quantity: "0.8",
 *       contractName: "contract name",
 *       tokenName: "token name"
 *     },
 *   ],
 *   erc721Tokens: [
 *     {
 *       contractAddress: "0x...",
 *       tokenId: "0",
 *       contractName: "contract name",
 *       tokenName: "token name"
 *     },
 *   ],
 *   erc1155Tokens: [
 *     {
 *       contractAddress: "0x...",
 *       tokenId: "1",
 *       quantity: "2",
 *       contractName: "contract name",
 *       tokenName: "token name"
 *     },
 *   ],
 * }
 * @typedef {Object} WrappedTokensOutput
 * @prop {ERC20WrappableOutput[]} erc20Tokens list of ERC20 tokens
 * @prop {ERC721WrappableOutput[]} erc721Tokens list of ERC721 tokens
 * @prop {ERC1155WrappableOutput[]} erc1155Tokens list of ERC1155 tokens
 */

/**
 * NFTMetadataOrUri
 *
 * Reference: {@link https://github.com/thirdweb-dev/typescript-sdk/blob/72b41bdc776bec81715f1d9e7cae548fd45a34c8/src/schema/tokens/common/index.ts NFTMetadataOrUri}
 * @typedef {string|Object} NFTMetadataOrUri
 * @prop {string|number} [name] name
 * @prop {string} [description] description
 * @prop {any} [image] image
 * @prop {any} [external_url] external_url
 * @prop {any} [animation_url] animation_url
 * @prop {string} [background_color] background_color
 * @prop {any} [properties] properties
 * @prop {any} [attributes] attributes
 * @prop {Json} [x:{string}] other optional propeties
 */

/**
 * NFTMetadata
 *
 * @typedef {Object} NFTMetadata
 * @prop {any} id id
 * @prop {string} uri uri
 * @prop {string|number} [name] name
 * @prop {string} [description] description
 * @prop {string} [image] image
 * @prop {string} [external_url] external url
 * @prop {string} [animation_url] animation url
 * @prop {Json} [x:{string}] other optional propeties
 */

/**
 * NFTMetadataOwner
 * @typedef {Object} NFTMetadataOwner
 * @prop {string} owner owner of contract
 * @prop {NFTMetadata} metadata metadata of contract
 */

/**
 * TransactionResult
 * @typedef {Object} TransactionResult
 * @prop {any} receipt receipt
 * @prop {Promise<unknown>} data data
 */

/**
 * TransactionResultWithId
 * @typedef {Object} TransactionResultWithId
 * @prop {BigNumber} id id
 * @prop {any} receipt receipt
 * @prop {Promise<unknown>} data data
 */

/**
 * Multiwrap
 *
 * @private
 * @typedef {Object} Multiwrap
 * @prop {any} key:{string} properties of Multiwrap
 */

/**
 * NFTContractDeployMetadata
 *
 * @typedef {Object} NFTContractDeployMetadata
 * @prop {string} name name of the contract
 * @prop {string} [description] description of the contract
 * @prop {any} [image] image for the contract
 * @prop {string} [external_link] url for the contract
 * @prop {string} [symbol] symbol for the NFTs
 * @prop {string[]} [trusted_forwarders] custom gasless trusted forwarder addresses
 * @prop {string} primary_sale_recipient the address that will receive the proceeds from primary sales
 * @prop {string} [fee_recipient] the address that will receive the proceeds from secondary sales (royalties)
 * @prop {number} [seller_fee_basis_points] the percentage (in basis points) of royalties for secondary sales
 * @prop {string} [platform_fee_recipient] the address that will receive the proceeds from platform fees
 * @prop {number} [platform_fee_basis_points] the percentage (in basis points) of platform fees
 */

/**
 * TokenContractDeployMetadata
 *
 * @typedef {Object} TokenContractDeployMetadata
 * @prop {string} name name of the contract
 * @prop {string} [description] description of the contract
 * @prop {any} [image] image for the contract
 * @prop {string} [external_link] url for the contract
 * @prop {string} [symbol] symbol for the NFTs
 * @prop {string[]} [trusted_forwarders] custom gasless trusted forwarder addresses
 * @prop {string} primary_sale_recipient the address that will receive the proceeds from primary sales
 * @prop {string} [platform_fee_recipient] the address that will receive the proceeds from platform fees
 * @prop {number} [platform_fee_basis_points] the percentage (in basis points) of platform fees
 */

/**
 * HazamaTokenContractDeployMetadata
 *
 * @typedef {Object} HazamaTokenContractDeployMetadata
 * @prop {string} name name of the contract
 * @prop {string} [description] description of the contract
 * @prop {string} author author of the contract
 * @prop {string} path path of the contract
 * @prop {string} tokenVersion token version
 * @prop {number} decimals token decimals
 * @prop {string} [symbol] token symbol
 * @prop {string} primarySaleRecipient the address that will receive the proceeds from primary sales
 * @prop {string} [feeRecipient] the address that will receive the proceeds from secondary sales (royalties)
 * @prop {number} [sellerFeeBasisPoints] the percentage (in basis points) of royalties for secondary sales
 * @prop {string} [platformFeeRecipient] the address that will receive the proceeds from platform fees
 * @prop {number} [platformFeeBasisPoints] the percentage (in basis points) of platform fees
 */

/**
 * HazamaTransferTokenMetadata
 *
 * @typedef {Object} HazamaTransferTokenMetadata
 * @prop {string} path string
 * @prop {string} [amount] number of tokens to transfer
 * @prop {string} destination address to transfer the tokens to
 * @prop {string|number} [name] idOrAmount // nft_send
 */

/**
 * HazamaClaimMetadata
 *
 * @typedef {Object} HazamaClaimMetadata
 * @prop {string} path string
 * @prop {Amount} amount number of tokens to claim
 * @prop {string} address address of the wallet you want to claim
 * @prop {BigNumberish} token token id
 */

/**
 * ClaimConditionInput
 *
 * @typedef {Object} ClaimConditionInput
 * @prop {Date} [startTime] start time for the presale/public sale
 * @prop {string} [currencyAddress] currency address
 * @prop {string|number} [price] sale price
 * @prop {string|number} [maxQuantity] limit how many mints for sale
 * @prop {string} [quantityLimitPerTransaction] limit how many mints per transaction
 * @prop {BigNumber} [waitInSeconds] waiting time
 * @prop {string} [merkleRootHash] merkle root hash
 * @prop {string[]} [snapshot] limit minting to only certain addresses
 */

/**
 * HazamaClaimConditionInput
 *
 * @typedef {Object} HazamaClaimConditionInput
 * @prop {string} path path of the contract
 * @prop {BigNumberish} token the id of the NFT to set claim conditions on
 * @prop {Date} [startTime] start time for the presale/public sale
 * @prop {string|number} [price] sale price
 * @prop {string|number} [maxQuantity] limit how many mints for sale
 * @prop {string} [quantityLimitPerTransaction] limit how many mints per transaction
 * @prop {BigNumber} [waitInSeconds] waiting time
 */

/**
 * Input roles object
 *
 * @example
 * {
 *   admin: ["0x..."],
 *   minter: ["0x..."],
 *   transfer: ["0x..."],
 * }
 * @typedef {Object} RolesObject
 * @prop {string[]} admin list of wallet address with admin role
 * @prop {string[]} minter list of wallet address with minter role
 * @prop {string[]} transfer list of wallet address with transfer role
 */

/**
 * Property
 *
 * @typedef {Object} Property
 * @prop {string} trait_type trait_type
 * @prop {string} value value
 */

/**
 * ObjectProperty
 *
 * @typedef {Object} ObjectProperty
 * @prop {any} key:{string} property
 */

/**
 * Output for token detail
 * 
 * @typedef {Object} TokenDetail
 * @prop {string} [id] id
 * @prop {string} [tokenId] token id
 * @prop {string} [title] token title
 * @prop {string} [image] token image
 * @prop {boolean} [isThirdweb] get info by Thirdweb or not
 * @prop {string} [animation_url] animation url
 * @prop {string} [description] token description
 * @prop {string} [type] contract type
 * @prop {string} [contractAddress] contract addres
 * @prop {Property[]|ObjectProperty} [tokenProperties] token properties
 * @prop {string} [tokenStandard] token standard
 * @prop {boolean} [isTransferable] is transferable
 * @prop {number} [sellerFeeBasisPoints] the percentage (in basis points) of royalties
 * @prop {string|string[]} [owner] list of owner

/**
 * ContractUpdateMetadata
 * 
 * @typedef {Object} ContractUpdateMetadata
 * @prop {string} name name of the contract
 */

/**
 * HazamaFungibleTokenMintMetadata
 *
 * @typedef {Object} HazamaFungibleTokenMintMetadata
 * @prop {string} path path of the contract
 * @prop {string} amount number of tokens to mint
 * @prop {string} mintto address to mint the tokens to
 */

/**
 * HazamaNFTMintMetadata
 *
 * @typedef {Object} HazamaNFTMintMetadata
 * @prop {string} path path of the contract
 * @prop {string} name name of the token
 * @prop {string} [description] description of the token
 * @prop {any} [image] image for the token
 */

/**
 * HazamaMultiNFTMintMetadata
 *
 * @typedef {Object} HazamaMultiNFTMintMetadata
 * @prop {string} path path of the contract
 * @prop {Array.<HazamaNFTMintMetadata>} mints list of token metadata
 */

/**
 * ContractTypeResult
 *
 * @typedef {Object} ContractTypeResult
 * @prop {string} type contract type
 */

/**
 * ProjectMetadata
 *
 * @typedef {Object} ProjectMetadata
 * @prop {string} id project id
 * @prop {string} name project name
 * @prop {string} network project network
 *
 */

/**
 * Project<br/>
 * index 0 - path<br/>
 * index 1 - project metadata
 *
 * @typedef {Array.<string|ProjectMetadata>} Project
 */

/**
 * Transactions list
 *
 * @typedef {Object} TransactionsList
 * @prop {number} [key:{string}] transaction timestamp (key is transaction id)
 *
 */

/**
 * WalletInfo
 * @typedef {Object} WalletInfo
 * @prop {number} [key:{string}] transaction id (key is path)
 * @prop {TransactionsList} tsx transaction list
 *
 */

/**
 * Contract
 *
 * @typedef {Object} Contract
 * @prop {string} Address Address
 * @prop {string} Author Author
 * @prop {string} [ContractHash] ContractHash
 * @prop {string} ContractVersion ContractVersion
 * @prop {string} [Id] Id
 * @prop {string} Name Name
 * @prop {string} Type Type
 * @prop {string} [ContractTypes] ContractTypes
 * @prop {boolean} [Transferable] Transferable
 * @prop {any} [key:{string}] others
 */

/**
 * Token
 *
 * @typedef {Object} Token
 * @prop {any} [key:{string}] others
 */

/**
 * TokenClaimConditionInput
 *
 * @typedef {Object} TokenClaimConditionInput
 * @prop {Date} [startTime] start time for the presale/public sale
 * @prop {string} [currencyAddress] currency address
 * @prop {string|number} [price] sale price
 * @prop {string|number} [maxQuantity] limit how many mints for sale
 * @prop {string} [quantityLimitPerTransaction] limit how many mints per transaction
 * @prop {BigNumber} [waitInSeconds] waiting time
 * @prop {string} [merkleRootHash] merkle root hash
 * @prop {string[]} [snapshot] limit minting to only certain addresses
 * @prop {string|number} [availableSupply]
 * @prop {string|number} [currentMintSupply]
 * @prop {any} [currencyMetadata]
 */

/**
 * TokenClaimData
 * 
 * @typedef {Object} TokenClaimData
 * @prop {TokenClaimConditionInput} condition condition
 * /

/**
 * EVMToken
 * 
 * @typedef {Object} EVMToken
 * @prop {number} id id
 * @prop {string} name token name
 * @prop {string} owner owner
 * @prop {string} description token description
 * @prop {any} image image
 * @prop {number} owner_index owner index
 * @prop {string} type type
 * @prop {string} contractType contract type
 * @prop {string} contractId contract id
 * @prop {boolean} Transferable transferable
 * @prop {number} owned owned
 * @prop {number} decimals decimals
 * @prop {number} supply supply
 * @prop {Date} startTime start time
 * @prop {any} startTimeOrigin start time origin
 * @prop {string|number} availableSupply available supply
 * @prop {string|number} maxQuantity max quantity
 * @prop {string[]} snapshot limit minting to only certain addresses
 * @prop {TokenClaimData} claimData claim data
 * @prop {number} [claimedSupply] claimed Supply
 * @prop {number} [unclaimedSupply] unclaimed Supply
 * @prop {number} [totalSupply] total Supply
 */

/**
 * HazamaTokenInfo
 * 
 * @typedef {Object} HazamaTokenInfo
 * @prop {number} decimals decimals
 * /

/**
 * HazamaToken
 * 
 * @typedef {Object} HazamaToken
 * @prop {string} name token name
 * @prop {string} version version
 * @prop {string} author author
 * @prop {string} symbol symbol
 * @prop {string} type type
 * @prop {string} contractType contract type
 * @prop {string} address address
 * @prop {string} contracthash contract hash
 * @prop {any} totalSupply total supply
 * @prop {any} owned owned
 * @prop {Date} [startTime] start time
 * @prop {any} [startTimeOrigin] start time origin
 * @prop {string|number} [availableSupply] available supply
 * @prop {string|number} [maxQuantity] max quantity
 * @prop {string[]} [snapshot] limit minting to only certain addresses
 * @prop {HazamaTokenInfo} [info] token info
 */

/**
 * ClaimMetadata
 *
 * @typedef {Object} ClaimMetadata
 * @prop {NFTMetadata} metadata metadata
 * @prop {ClaimConditionInput} condition condition
 * @prop {boolean} canClaim be able to claim
 * @prop {string} claimReason claim reason
 * @prop {string} [path] path
 */

/**
 * HazamaContractUpdateMetadata
 *
 * @typedef {Object} HazamaContractUpdateMetadata
 * @prop {string} name name of the contract
 * @prop {string} [description] description of the contract
 * @prop {string} author author of the contract
 * @prop {string} path path of the contract
 * @prop {string} tokenVersion token version
 * @prop {number} decimals token decimals
 * @prop {string} [symbol] token symbol
 * @prop {string} primarySaleRecipient the address that will receive the proceeds from primary sales
 * @prop {string} [feeRecipient] the address that will receive the proceeds from secondary sales (royalties)
 * @prop {number} [sellerFeeBasisPoints] the percentage (in basis points) of royalties for secondary sales
 * @prop {string} [platformFeeRecipient] the address that will receive the proceeds from platform fees
 * @prop {number} [platformFeeBasisPoints] the percentage (in basis points) of platform fees
 */

/**
 * ApiRequest
 *
 * @typedef {Object} ApiRequest
 * @prop {string} apiCallStatus api call status
 * @prop {string} apiBody api body
 * @prop {string} apiURI api uri
 * @prop {string} apiMethod api method
 */

/**
 * ProposalExecution
 *
 * @typedef {Object} ProposalExecution
 * @prop {string} toAddress the contract you want to make a call to
 * @prop {number} nativeTokenValue the amount of the native currency to send in this transaction
 * @prop {string} transactionData transaction data that will be executed when the proposal is executed
 */
